from vbelts import speed as s
import pytest

def eval_peripheral(diam, rpm):
    return s.peripheral(diam, rpm)


def test_peripheral():
    assert eval_peripheral(240, 1750) == 21.99114857512855


def test_peripheral_fail():
    with pytest.raises(Exception):
        eval_peripheral('a', 100)


from vbelts import length as l
import pytest

# PulleyBelt lc

def eval_pulleybelt_lc(min_diam, max_diam, belt_model, b_profile):
    return l.PulleyBelt(min_diam, max_diam, belt_model, b_profile).l_c()


def test_pulleybelt_lc():
    assert eval_pulleybelt_lc(120, 240, 'HiPower', 'a') == (1200, 'A-46')

# PulleyBelt cc

def eval_pulleybelt_ca(min_diam, max_diam, belt_model, b_profile):
    return l.PulleyBelt(min_diam, max_diam, belt_model, b_profile).c_c()


def test_pulleybelt_ca():
    assert eval_pulleybelt_ca(120, 240, 'HiPower', 'a') == 310.72814411208714


import csv
import os
import re
from abc import ABC, abstractmethod

mach_group_data = [
    ['stirrer', '1'],
    ['small blower', '1'],
    ['exhaustor', '1'],
    ['centrifugal pump', '1'],
    ['regular compressor', '1'],
    ['light conveyor belt', '1'],
    ['heavy conveyor belt', '2'],
    ['large blower', '2'],
    ['generator', '2'],
    ['transmission axle', '2'],
    ['laundry machine', '2'],
    ['press', '2'],
    ['graphical machine', '2'],
    ['positive displacement pump', '2'],
    ['sieving machine', '2'],
    ['pottery machine', '3'],
    ['bucket elevator', '3'],
    ['reciprocating compressor', '3'],
    ['mill', '3'],
    ['carpentry machine', '3'],
    ['textile machine', '3'],
    ['crusher', '4'],
    ['crane', '4'],
    ['tire shop machine', '4'],
]

drive_group_data = [
    ['normal torque ac', '1'],
    ['ring cage ac', '1'],
    ['synchronous ac', '1'],
    ['phase division ac', '1'],
    ['derivation dc', '1'],
    ['multiple cylinders combustion', '1'],
    ['high torque ac', '2'],
    ['high slipping ac', '2'],
    ['repulsion induction ac', '2'],
    ['monophasic ac', '2'],
    ['series winding dc', '2'],
    ['collector rings ac', '2'],
    ['mixed winding dc', '2'],
    ['single cylinder combustion', '2'],
    ['transmission axle', '2'],
    ['clutch', '2'],
]

class _OutOfRangeError(Exception):
    """Raised when value is out of range of the list"""
    pass

class _NotValidError(Exception):
    """Raised when the value passed to the function is not valid"""
    pass


class _ConvergenceError(Exception):
    """Raised when the value passed to an iterator function does not converge"""
    pass


class _Interpolate():
    """Interpolate class linear interpolate values.
    
    Parameters
    ----------
    x_data : float
        Input data point to be calculated, [-]
    x_min : float
        Nearest minimum input value in relation to the data point, [-]
    x_max : float
        Nearest maximum input value in relation to the data point, [-]
    y_min : float
        Nearest minimum output value in relation to the data point, [-]
    y_max : float
        Nearest maximum output value in relation to the data point, [-]
    """
    def __init__(self, x_data, x_min, x_max, y_min, y_max):
        self.x_data = x_data
        self.x_min = x_min
        self.x_max = x_max
        self.y_min = y_min
        self.y_max = y_max
    
    def y_data(self):
        """Linear interpolation of table data

        Returns
        -------
        x : float
            Interpolated output data point
        """
        return self.y_max-((self.x_max - self.x_data)/(self.x_max - self.x_min))*(self.y_max - self.y_min)


class _MinDist():
    """MinDist class selects the nearest data value from a point in between.

    Parameters
    ----------
    x : float
        Value on the interval, [-]
    x_min : float
        Nearest minimum value on the scale, [-]
    x_max : float
        Nearest maximum value on the scale, [-]
    """
    def __init__(self, x:float, x_min:float, x_max:float):
        self.x = x
        self.x_min = x_min
        self.x_max = x_max
    
    def calc(self):
        """Nearest value between two items given one value that is in the interval.

        Returns
        -------
        x_nearest : float
            Nearest value of x
        """
        dist_1 = (self.x - self.x_min)/self.x
        dist_2 = (self.x_max - self.x)/self.x
        if dist_1 < dist_2:
            return self.x_min
        elif dist_1 > dist_2:
            return self.x_max
        elif dist_1 == dist_2: # if both are equal, return the maximum
            return self.x_max
        else:
            raise ValueError


class _ReIterate():
    def __init__(self, interpol:_Interpolate=_Interpolate, m_dist:_MinDist=_MinDist):
        self.interpol = interpol
        self.m_dist = m_dist


    def read(self, dict_master_list):
        for line in dict_master_list:
            yield line

    def three_rows(self, dict_list, param_1, param_2):
        """_Iterate through a list with a list of three columns using two parameters and return the last one, _Interpolated or not
        """
        last_row_1 = 0
        last_row_2 = 0
        for line in self.read(dict_list):
            if float(line['diameter']) == param_1:
                if float(line['rpm']) == param_2:
                    return (float(line['power_b']), False)  # first return the value, second need for adjusting, used in TransPower._basic()
                elif float(line['rpm']) > param_2:
                    return (self.interpol(param_2, last_row_1, float(line['rpm']), last_row_2, float(line['power_b'])).y_data(), False)
            elif float(line['diameter']) > param_1:
                if float(line['rpm']) == param_2:
                    return (float(line['power_b']), True)
                elif float(line['rpm']) > param_2:
                    return (self.interpol(param_2, last_row_1, float(line['rpm']), last_row_2, float(line['power_b'])).y_data(), True)
            last_row_1 = float(line['rpm'])
            last_row_2 = float(line['power_b'])
        raise _OutOfRangeError('Value out of range for these parameters')


    def four_rows(self, dict_list, param_1, param_2):
        """Iterate through a list of four columns using two parameters, the first two are a range, and return the last, _Interpolated or not."""
        for line in self.read(dict_list):
            if float(line['gr_low']) > param_1:
                raise _OutOfRangeError('Value out of range for these parameters')
            elif float(line['gr_low']) <= param_1 < float(line['gr_high']):
                if float(line['rpm']) == param_2:
                    return float(line['power_a'])
                elif float(line['rpm']) > param_2:
                    return self.interpol(param_2, last_row_2, float(line['rpm']), last_row_3, float(line['power_a'])).y_data()
            last_row_2 = float(line['rpm'])
            last_row_3 = float(line['power_a'])
        raise _OutOfRangeError('Value out of range for these parameters')


    def three_rows_choose(self, dict_list, param, w_column:str):
        """_Iterate through a file with a list of three rows using one parameter, chosing which row to use to return the value.
        """
        last_row_1 = 0
        last_row_chosed = 0
        for line in self.read(dict_list):
            if float(line['factor']) == param:
                return float(line[w_column])
            elif float(line['factor']) > param:
                return self.interpol(param, last_row_1, float(line['factor']), last_row_chosed, float(line[w_column])).y_data()
            last_row_1 = float(line['factor'])
            last_row_chosed = float(line[w_column])
        raise _OutOfRangeError('Value out of range for these parameters')


    def belt_type(self, dict_list:list, param_1:str, param_2:float):
        for line in self.read(dict_list):
            if line['profile'] == param_1:
                last_length = 0
                if float(line['length']) == param_2:  # if length == length on data
                    return (float(line['length']), line['type'])
                elif float(line['length']) > param_2:
                    chosed_length = self.m_dist(param_2, last_length, float(line['length'])).calc()
                    # check what length and type was chosen and return them
                    if chosed_length == float(line['length']):
                        chosed_type = line['type']
                    else:
                        chosed_type = last_type
                    return (chosed_length, chosed_type)
            last_length = float(line['length'])
            last_type = line['type']
        raise _OutOfRangeError('Value out of range for these')

class _CSV(ABC):
    """Abstract class for handling csv files"""
    @abstractmethod
    def read(self): pass


class _Iterate(_CSV):
    """Iterate class is responsible for iterating through the multiple csv files in the package.
    
    Parameters
    ----------
    filename : str
        Filename of the csv file without the extension, [-]
    row : str
        Rows of data of the csv file, [-]
    """
    def __init__(self, filename:str, *row:str, interpol:_Interpolate=_Interpolate, m_dist:_MinDist=_MinDist):
        self.filename = filename
        self.row = row
        self.interpol = interpol
        self.m_dist = m_dist

    def read(self):
        """Read csv data and return a generator function.
        
        Returns
        -------
        x : generator
            Generator data, [-]
        """
        file_path = os.path.join(os.path.dirname(__file__), 'data', self.filename)  # fetch full path
        with open(f'{file_path}.csv', mode='r', encoding="utf-8") as csv_file:
            reader = csv.DictReader(csv_file, quoting=csv.QUOTE_NONNUMERIC)  # not quoted values are floats
            # next(reader)
            for line in reader:
                yield line
    

    def fcc(self, vbelt_model:str, vbelt_type:str):
        self.filename = f'{vbelt_model}_fcc'
        for line in self.read():
            if line['type'] == vbelt_type:
                return float(line['fcc'])

    
    def three_rows(self, *param:float):
        """_Iterate through a file with a list of three rows using two parameters and return the last one, _Interpolated or not
        """
        for line in self.read():
            if line[self.row[0]] == param[0]:
                if line[self.row[1]] == param[1]:
                    return (line[self.row[2]], False)  # first return the value, second need for adjusting, used in TransPower._basic()
                elif line[self.row[1]] > param[1]:
                    return (self.interpol(param[1], last_row_1, line[self.row[1]], last_row_2, line[self.row[2]]).y_data(), False)
            elif line[self.row[0]] > param[0]:
                if line[self.row[1]] == param[1]:
                    return (self.interpol(param[1], last_row_1, line[self.row[1]], last_row_2, line[self.row[2]]).y_data(), True)
                elif line[self.row[1]] > param[1]:
                    return (self.interpol(), True)
            last_row_1 = line[self.row[1]]
            last_row_2 = line[self.row[2]]
        raise _OutOfRangeError('Value out of range for these parameters')


    def three_rows_choose(self, param, w_row:float):
        """_Iterate through a file with a list of three rows using one parameter, chosing which row to use to return the value.
        """
        w_row -= 1
        last_row_1 = 0
        last_row_chosed = 0
        for line in self.read():
            if line[self.row[0]] == param:
                return line[self.row[w_row]]
            elif line[self.row[0]] > param:
                return self.interpol(param, last_row_1, line[self.row[0]], last_row_chosed, line[self.row[w_row]]).y_data()
            last_row_1 = line[self.row[0]]
            last_row_chosed = line[self.row[w_row]]
        raise _OutOfRangeError('Value out of range for these parameters')


    def four_rows(self, *param:float):
        """_Iterate through a file with a list of four rows using two parameters, the first two are a range, and return the last, _Interpolated or not.
        """
        for line in self.read():
            if line[self.row[0]] > param[0]:
                raise _OutOfRangeError('Value out of range for these parameters')
            elif line[self.row[0]] <= param[0] < line[self.row[1]]:
                if line[self.row[2]] == param[1]:
                    return line[self.row[3]]
                elif line[self.row[2]] > param[1]:
                    return self.interpol(param[1], last_row_2, line[self.row[2]], last_row_3, line[self.row[3]]).y_data()
            last_row_2 = line[self.row[2]]
            last_row_3 = line[self.row[3]]
        raise _OutOfRangeError('Value out of range for these parameters')

    
    def belt_type(self, *param:float):
        for line in self.read():
            if line[self.row[0]] == param[0]:
                last_length = 0
                if line[self.row[1]] == param[1]:  # if length == length on data
                    return (line[self.row[1]], line[self.row[2]])
                elif line[self.row[1]] > param[1]:
                    chosed_length = self.m_dist(param[1], last_length, line[self.row[1]]).calc()
                    # check what length and type was chosen and return them
                    if chosed_length == line[self.row[1]]:
                        chosed_type = line[self.row[2]]
                    else:
                        chosed_type = last_type
                    return (chosed_length, chosed_type)
            last_length = line[self.row[1]]
            last_type = line[self.row[2]]
        raise _OutOfRangeError('Value out of range for these')


class _Device():
    r"""Device class creates the blueprint for the Motor and Machine classes.

    Parameters
    ----------
    name : str
        Device name, [-]
    li_group : list
        Device group list, [-]

    Attributes
    ----------
    group : int
        Device group classification, [-]
    """
    def __init__(self, name:str, li_group:list):
        self.name = name
        self.li_group = li_group
        self.group = self._grp()
    

    def _grp(self):
        r"""Classifies the device group based on the name using the self.li_group.

        Returns
        -------
        group : int
            Device group
        """
        data_regex = re.compile(r'(\w+).?(\w+)?.?(\w+)?')
        try:
            for item in self.li_group:
                m = data_regex.search(item[0])  # search the first item
                if self.name in m.group(0):
                    return int(item[1])
        except:  # stop the execution if nothing is found
            raise ValueError


class Motor(_Device):
    r"""Motor class centralizes and classifies engine properties. For valid motor data entries see the :ref:`Data <motor_machine_data>` section.

    Parameters
    ----------
    name : str
        Motor name, [-]
    power : float
        Motor power, [hp]

    Attributes
    ----------
    group : int
        Motor group classification, [-]
    
    Examples
    --------
    >>> engine = vbelts.util.Motor('multiple cylinders', 3)
    >>> engine.group
    1
    >>> engine.power
    3
    
    Notes
    -----
    The group classification is needed for the service factor [#]_ calculation.

    References
    ----------
    .. [#] Oleostatic. 2016. "OLEOSTATIC Correas Trapeciales Convencionales", **Universidad Carlos III de Madrid**. Accessed September 23, 2020. http://ocw.uc3m.es/ingenieria-mecanica/diseno-mecanico-1/material_clase/ocw_catalogo_correas.
    """
    def __init__(self, name:str, power:float):
        super().__init__(name, li_group=drive_group_data)
        self.power = power
        self.li_group = drive_group_data


class Machine(_Device):
    r"""Machine class centralizes and classifies engine properties. For valid machine data entries see the :ref:`Data <motor_machine_data>` section.

    Parameters
    ----------
    name : str
        Machine name, [-]
    hours_service : float
        Hours of service per day, [h/day]

    Attributes
    ----------
    group : int
        Machine group classification, [-]
    
    Examples
    --------
    >>> mach = vbelts.util.Machine('reciprocating compressor', 18)
    >>> mach.group
    3
    >>> mach.hours_service
    18
    
    Notes
    -----
    The group classification is needed for the service factor [#]_ calculation.

    References
    ----------
    .. [#] Oleostatic. 2016. "OLEOSTATIC Correas Trapeciales Convencionales", **Universidad Carlos III de Madrid**. Accessed September 23, 2020. http://ocw.uc3m.es/ingenieria-mecanica/diseno-mecanico-1/material_clase/ocw_catalogo_correas.
    """
    def __init__(self, name:str, hours_service:float):
        super().__init__(name, li_group=mach_group_data)
        self.hours_service = hours_service
        self.li_group = mach_group_data


class _Belt():
    """Belt is a blueprint class for the HiPower and SuperHC v-belt classes"""
    def __init__(self, est_power:float, rpm_fastest:float):
        self.est_power = est_power
        self.rpm_fastest = rpm_fastest


    def _fun_val(self, a:float, b:float, c:float, x:float, upper:float):
        """X for a mathematical function with a linear (ax + b) and a constant part.


        :param a: a coefficient for the linear part of the function
        :type a: float
        :param b: b coefficient for the linear part of the function
        :type b: float
        :param c: y value for the constant part of the function
        :type c: float
        :param x: input value of x
        :type x: float
        :param upper: upper limit of the linear function in the y axis
        :type upper: float
        :return: y value for the math function
        :rtype: float
        """
        if x < upper:
            eq = a * x + b
            return abs(eq)
        # if the value is within the constant range
        elif x >= upper:
            eq = c
            return eq


class _Pulley(ABC):
    """Abstract class for the Driven and Driving pulley class"""
    def __init__(self, diam, vbelt_profile:str, power:float, rpm:float, iterator:_ReIterate=_ReIterate):
        self.diam = diam
        self.vbelt_profile = vbelt_profile
        self.power = power
        self.rpm = rpm
        self.filename = f'{self.vbelt_profile}_diam'
        self.iterator = iterator

    # @abstractmethod
    # def min_diam(self): pass


def gear_ratio(rpm_input:float, rpm_output:float):
    r"""Pulley gear ratio based on the rpm input and output.

    Parameters
    ----------
    rpm_input : float
        input (driving) axle speed, [rpm]
    rpm_output : float
        output (driven) axle speed, [rpm]
    
    Returns
    -------
    gear_ratio : float
        Calculated gear ratio, [-]
    
    Examples
    --------
    >>> vbelts.util.gear_ratio(1000, 1750)
    0.5714285714285714
    
    Notes
    -----
    The rpm based gear ratio is calculated [#]_ as:

    .. math::
        R = \frac{N_{in}}{N_{out}}
    
    References
    ----------
    .. [#] Douglas Wright. 2005."DANotes: V-Belt drives: Introduction", **V-BELT DRIVES**. Accessed September 23, 2020, http://www-mdp.eng.cam.ac.uk/web/library/enginfo/textbooks_dvd_only/DAN/V-belts/intro/intro.html.
    """
    if isinstance(rpm_input, (float, int)) and isinstance(rpm_output, (float, int)):
        return rpm_input/rpm_output
    else:
        raise ValueError



from vbelts import pulley as p


# Driven

def eval_driven_driving(diam, b_profile, power, rpm, gear_ratio):
    return p.Driven(diam, b_profile, power, rpm, gear_ratio).driving_pulley()


def test_driven_driving():
    assert eval_driven_driving(240, 'a', 3, 1750, 1.846) == 130.01083423618635


def eval_driven_commercial(diam, b_profile, power, rpm, gear_ratio, driving_pulley, desired_pulley_diam):
    return p.Driven(diam, b_profile, power, rpm, gear_ratio).commercial(driving_pulley, desired_pulley_diam)


def test_driven_commercial():
    assert eval_driven_commercial(245, 'a', 3, 1750, 1.846, 130, 120) == [240, 1.8461538461538463, 1750, 947.9166666666666]


# Driving

def eval_driving(diam, b_profile, power, rpm, gear_ratio):
    return p.Driving(diam, b_profile, power, rpm, gear_ratio).driven_pulley()


def test_driving():
    assert eval_driving(130, 'a', 3, 1000, 1.846) == 239.98000000000002


from vbelts import belt as b
import pytest

# HiPower Class

def eval_hipower(power, rpm):
    return b.HiPower(power, rpm).profile


def test_hipower():
    assert eval_hipower(3,500) == 'a'
    assert eval_hipower(70, 4000) == 'a'
    assert eval_hipower(9, 400) == 'b'
    assert eval_hipower(60, 870) == 'c'
    assert eval_hipower(90, 400) == 'd'


def test_hipower_fail():
    with pytest.raises(Exception):
        eval_hipower('a', 1000)
        eval_hipower(3, 'a')
        eval_hipower(-3, 500)
        eval_hipower(3, -500)
        eval_hipower(3, 5001)
        eval_hipower(501, 150)
        eval_hipower(3, 99)
        eval_hipower(0.9, 500)
        eval_hipower(1, None)
        eval_hipower(None, 150)


# SuperHC Class

def eval_superhc(power, rpm):
    return b.SuperHC(power, rpm).profile


def test_superhc():
    assert eval_superhc(4, 1160) == '3v'
    assert eval_superhc(30, 690) == '5v'
    assert eval_superhc(150, 575) == '8v'


def test_superhc_fail():
    with pytest.raises(Exception):
        eval_superhc('a', 1000)
        eval_superhc(3, 'a')
        eval_superhc(-3, 500)
        eval_superhc(3, -500)
        eval_superhc(0, 500)
        eval_superhc(3, 5001)
        eval_superhc(1001, 150)
        eval_superhc(3, 99)
        eval_superhc(0.9, 500)


from vbelts import power as p
import pytest

# EstPower

def eval_estpower(power, d_group, m_group, h_service):
    return p.EstPower(power, d_group, m_group, h_service).calc()


def test_estpower():
    # drive group 1
    assert eval_estpower(2, 1, 1, 4) == 2
    assert eval_estpower(2, 1, 1, 8) == 2.2
    assert eval_estpower(2, 1, 1, 18) == 2.4
    assert eval_estpower(2, 1, 2, 4) == 2.2
    assert eval_estpower(2, 1, 2, 8) == 2.4
    assert eval_estpower(2, 1, 2, 18) == 2.6
    assert eval_estpower(2, 1, 3, 4) == 2.4
    assert eval_estpower(2, 1, 3, 8) == 2.6
    assert eval_estpower(2, 1, 3, 18) == 2.8
    assert eval_estpower(2, 1, 4, 4) == 2.6
    assert eval_estpower(2, 1, 4, 8) == 2.8
    assert eval_estpower(2, 1, 4, 18) == 3
    # drive group 2
    assert eval_estpower(2, 2, 1, 4) == 2.2
    assert eval_estpower(2, 2, 1, 8) == 2.4
    assert eval_estpower(2, 2, 1, 18) == 2.6
    assert eval_estpower(2, 2, 2, 4) == 2.4
    assert eval_estpower(2, 2, 2, 8) == 2.6
    assert eval_estpower(2, 2, 2, 18) == 2.8
    assert eval_estpower(2, 2, 3, 4) == 2.8
    assert eval_estpower(2, 2, 3, 8) == 3
    assert eval_estpower(2, 2, 3, 18) == 3
    assert eval_estpower(2, 2, 4, 4) == 3.2
    assert eval_estpower(2, 2, 4, 8) == 3.2
    assert eval_estpower(2, 2, 4, 18) == 3.6


def test_estpower_fail():
    with pytest.raises(Exception):
        eval_estpower('a', 1, 1, 3)
        eval_estpower(0, 1, 1, 3)
        eval_estpower(1, 0, 1, 3)
        eval_estpower(1, 1, 0, 3)
        eval_estpower(1, 1, 1, 2)


# TransPower

def eval_transpower(b_model, b_profile, b_type, est_power, g_ratio, l_corr, min_diam, max_diam, rpm):
    return p.TransPower(b_model, b_profile, b_type, est_power, g_ratio, l_corr, min_diam, max_diam, rpm).belt_qty()


def test_transpower():
    assert eval_transpower('HiPower', 'a', 'A-32', 2, 130/240,850, 130, 240, 1750) == 0.5060451558976288


def test_transpower_fail():
    with pytest.raises(Exception):
        eval_transpower(1, 'a', 'A-32', 2, 120/240, 850, 130, 240, 1750)


from math import pi

def peripheral(pulley_diam:float, pulley_rpm:float):
    r"""Mean peripheral linear velocity of the v-belt.


    Parameters
    ----------
    pulley_diam : float
        mean diameter of the pulley, [mm]
    pulley_rpm : float
        rotational speed of the pulley, [rpm]
    
    Returns
    -------
    v_peripheral : float
        Linear velocity of the v-belt, [m/s]
    

    Examples
    --------
    >>> vbelts.speed.peripheral(240, 1750):
    21.99114857512855


    Notes
    -----
    The velocity is defined [#]_ as:

    .. math::
        v_p &= \omega \cdot r \\
        v_p &= \frac{\pi \cdot r \cdot n}{30}


    References
    ----------
    .. [#] Claudino Alves, Claudemir. "Transmissão por Correias - Dimensionamento Atividade 2". **Fatec Itaquera**. Accessed September 16, 2020, http://claudemiralves.weebly.com/uploads/3/8/6/2/3862918/dimen._de_correias.pdf
    """
    pulley_diam_m = pulley_diam/1000
    return (pi * (pulley_diam_m/2) * pulley_rpm)/30


from vbelts.util import _Pulley, _Iterate, _ReIterate, _ConvergenceError
from time import time

class Driven(_Pulley):
    r"""Driven class stores essential properties of the driven pulley and calculates driving pulley properties. It also can select the appropriate commercial diameter for the driven pulley, updating the relevant properties like output rpm and gear ratio.

    Parameters
    ----------
    diam : float
        Diameter of the driven pulley, [mm]
    vbelt_profile : str
        Profile of the v-belt, [-]
    power : float
        Power of the system, [hp]
    rpm : float
        Rotational speed of the driven axle, [rpm]
    gear_ratio : float
        Gear ratio of the pulleys in decimal form, [-]
    
    Examples
    --------
    >>> driven_pulley = vbelts.pulley.Driven(240, 'a', 3, 1750, 1.846)
    >>> driven_pulley.driving_pulley()
    130.01083423618635
    >>> driven_pulley.commercial(130, 120)
    [240, 1.8461538461538463, 1750, 947.9166666666666]
    """
    def __init__(self, diam:float, vbelt_profile:str, power:float, rpm:float, gear_ratio:float, iterator:_ReIterate=_ReIterate):
        super().__init__(diam, vbelt_profile, power, rpm, iterator)
        self.gear_ratio = gear_ratio
    

    def driving_pulley(self):
        r"""Calculated driving pulley diameter.

        Returns
        -------
        driving_diam : float
            Driving pulley diameter, [mm]
        
        Notes
        -----
        The driven gear ratio is calculated [#]_ by:

        .. math::
            D_{driving} = \frac{D_{driven}}{R}
        
        Where the `R` is the gear ratio.

        References
        ----------
        .. [#] Douglas Wright. 2005."DANotes: V-Belt drives: Introduction", **V-BELT DRIVES**. Accessed September 23, 2020, http://www-mdp.eng.cam.ac.uk/web/library/enginfo/textbooks_dvd_only/DAN/V-belts/intro/intro.html.
        """
        return self.diam / self.gear_ratio
    

    def commercial(self, driving_pulley:float, desired_pulley_diam:int):
        r"""Selects an approximate to the commercial diameter of a pulley through iterator convergence.

        Returns
        -------
        result : list
            A list containing the respective results, [-]
        diam_out : float
            Calculated commercial driven pulley diameter, [mm]
        gear_ratio_comm : float
            Calculated gear ratio for the commercial diameter, [-]
        rpm : float
            Input rotational speed, [rpm]
        rpm_out : float
            Calculated output rotational speed for the commercial driven diameter, [rpm]
        """
        # calculate gear ratio range
        range_rpm = 100
        rpm_output = self.rpm/self.gear_ratio
        gear_ratio_max = self.rpm/(rpm_output - range_rpm)
        gear_ratio_min = self.rpm/(rpm_output + range_rpm)
        error_min = 0.01
        driven_pulley = self.gear_ratio * driving_pulley
        diam_calc = driven_pulley

        # _Iterate
        error = 1
        t_i = time()
        while error > error_min:
            gear_ratio_calc = diam_calc/driving_pulley
            # control iteration parameters
            if gear_ratio_calc > gear_ratio_max:
                diam_calc -= diam_calc * error * 0.01
            elif gear_ratio_calc < gear_ratio_min:
                diam_calc += diam_calc * error
            else:
                diam_calc *= error
            error = 1-(diam_calc/desired_pulley_diam)
            # control for infinite loops
            t_f = time()
            elapsed_time = t_f - t_i
            if elapsed_time > 4:
                raise _ConvergenceError('The function could not reach convergence, try again with a lower value')
        
        # recalculate rpm_output
        base = 10
        diam_out = base * round(diam_calc/base)  # round the number to nearest 5 integer
        gear_ratio_comm = diam_out/driving_pulley
        rpm_out = self.rpm/gear_ratio_comm
        return [diam_out, gear_ratio_comm, self.rpm, rpm_out]



class Driving(_Pulley):
    r"""Driving class stores essential properties of the driving pulley and calculates driven pulley properties.

    Parameters
    ----------
    diam : float
        Diameter of the driving pulley, [mm]
    vbelt_profile : str
        Profile of the v-belt, [-]
    power : float
        Power of the system, [hp]
    rpm : float
        Rotational speed of the driving axle, [rpm]
    gear_ratio : float
        Gear ratio of the pulleys in decimal form, [-]
    
    Examples
    --------
    >>> driving_pulley = vbelts.pulley.Driving(130, 'a', 3, 1000, 1.846)
    >>> driving_pulley.driven_pulley()
    239.98000000000002
    """
    def __init__(self, diam:float, vbelt_profile:str, power:float, rpm:float, gear_ratio:float, iterator:_Iterate=_Iterate):
        super().__init__(diam, vbelt_profile, power, rpm, iterator)
        self.gear_ratio = gear_ratio

    def driven_pulley(self):
        r"""Calculated driven pulley diameter.

        Returns
        -------
        driven_diam : float
            Driven pulley diameter, [mm]
        
        Notes
        -----
        The driven gear ratio is calculated [#]_ by:

        .. math::
            D_{driven} = D_{driving} \cdot R
            
        Where the `R` is the gear ratio.

        References
        ----------
        .. [#] Douglas Wright. 2005."DANotes: V-Belt drives: Introduction", **V-BELT DRIVES**. Accessed September 23, 2020, http://www-mdp.eng.cam.ac.uk/web/library/enginfo/textbooks_dvd_only/DAN/V-belts/intro/intro.html.
        """
        return self.diam * self.gear_ratio


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    # Mandatory and initial setup
    name='vbelts',
    version='0.3.9',
    description='Utilities for v-belt dimensioning',
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    package_data={'':["data/*.csv"]},
    include_package_data=True,
    author='Glademir Karpinski Junior, Hector Balke Nodari',
    author_email='gkarpinskijr@gmail.com, hectornodari@gmail.com',

    # Secondary setup
    url='https://github.com/traumatic-monotype/vbelts',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Natural Language :: English',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        ],
    extras_require = {"dev": ["pytest==6.0.1","check-manifest==0.42","sphinx==3.2.1","autodoc==0.5.0","sphinx-rtd-theme==0.5.0"]},
)



from vbelts.util import _Belt, _OutOfRangeError

class HiPower(_Belt):
    r"""HiPower class checks the conditions and selects the v-belt profile for this model.

    Parameters
    ----------
    est_power : float
        estimated power of the system, [hp]
    rpm_fastest : float
        fastest rotational speed of the system, [rpm]
    

    Attributes
    ----------
    profile : str
        Selected v-belt profile, [-]
    

    Examples
    --------
    >>> belt = vbelts.belt.HiPower(3, 500)
    >>> belt.profile
    a
    >>> belt1 = vbelts.belt.HiPower(9, 400)
    >>> betl1.profile
    b


    Notes
    -----
    The data [#]_ is available online.


    References
    ----------
    .. [#] "CLASSICAL," V-Belts, BestTORQ, accessed September 21, 2020,  https://www.bestorq.com/Library/media/CLASSICAL_xselect.gif
    """
    def __init__(self, est_power:float, rpm_fastest:float):
        super().__init__(est_power, rpm_fastest)
        self._boundary_a = super(HiPower, self)._fun_val(74.1950272674027, 47.215442190042, 3322, self.est_power, 50.7)
        self._boundary_b = super(HiPower, self)._fun_val(19.3889694765281, 32.2532889843209, 2151.4, self.est_power, 122.3)
        self._boundary_c = super(HiPower, self)._fun_val(4.94622293507244, 19.7301462298106, 1335.9, self.est_power, 277.35)
        self._belt_profile()
    
    
    def _belt_profile(self):
        r"""Method selects the appropriate profile based on the region limits.
         """
        # Define the maximum range
        if self.rpm_fastest > 5000 or self.est_power > 500:
            return _OutOfRangeError('Values out of range for the HiPower model: rpm > 5000 or est_power > 500')
        elif self.rpm_fastest < 100 or self.est_power < 1:
            return _OutOfRangeError('Values out of range for the HiPower model: rpm < 100 or est_power < 1')

        if self.rpm_fastest >= self._boundary_a:
            profile = 'a'
        elif self._boundary_b <= self.rpm_fastest < self._boundary_a:
            profile = 'b'
        elif self._boundary_c <= self.rpm_fastest < self._boundary_b:
            profile = 'c'
        elif self._boundary_c > self.rpm_fastest:
            profile = 'd'
        self.profile = profile


class SuperHC(_Belt):
    r"""SuperHC class calculates checks the conditions and selects the v-belt profile for this model.

    Parameters
    ----------
    est_power : float
        estimated power of the system, [hp]
    rpm_fastest : float
        fastest rotational speed of the system, [rpm]
    

    Attributes
    ----------
    profile : str
        Selected v-belt profile, [-]
    

    Examples
    --------
    >>> belt = vbelts.belt.SuperHC(4, 1160)
    >>> belt.profile
    3v
    >>> belt1 = vbelts.belt.SuperHC(30, 690)
    >>> betl1.profile
    5v


    Notes
    -----
    The data [#]_ is available online.


    References
    ----------
    .. [#] "WEDGE," V-Belts, BestTORQ, accessed September 21, 2020,  https://www.bestorq.com/Library/media/wedge_xselect358.gif
    """
    def __init__(self, est_power:float, rpm_fastest:float):
        super().__init__(est_power, rpm_fastest)
        self._boundary_3v = super(SuperHC, self)._fun_val(40.6961726224751, 11.8866879052094, 3316.25, self.est_power, 91)
        self._boundary_5v = super(SuperHC, self)._fun_val(4.30114168431602, 3.84423100031302, 1332.74, self.est_power, 309)
        self._belt_profile()

    
    def _belt_profile(self):
        r"""Method selects the appropriate profile based on the region limits.
         """
        # Define the maximum range
        if self.rpm_fastest > 5000 or self.est_power > 1000:
            return _OutOfRangeError('Values out of range for the SuperHC model: rpm > 5000 or est_power > 1000')
        elif self.rpm_fastest < 100 or self.est_power < 1:
            return _OutOfRangeError('Values out of range for the SuperHC model: rpm < 100 or est_power < 1')
        
        if self.rpm_fastest >= self._boundary_3v:
            profile = '3v'
        elif self._boundary_5v <= self.rpm_fastest < self._boundary_3v:
            profile = '5v'
        elif self._boundary_5v > self.rpm_fastest:
            profile = '8v'
        self.profile = profile
